@@ -9,13 +9,14 @@ import (
 
 	"github.com/argoproj/gitops-engine/pkg/utils/kube"
 	"github.com/argoproj/pkg/errors"
-	"github.com/ghodss/yaml"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/apis/meta/v1/unstructured"
+	"sigs.k8s.io/yaml"
 
 	"github.com/argoproj/argo-cd/v2/pkg/apis/application/v1alpha1"
+	"github.com/argoproj/argo-cd/v2/util/argo/normalizers"
 )
 
 func strToUnstructured(jsonStr string) *unstructured.Unstructured {
@@ -52,7 +53,7 @@ var (
     resourceVersion: "123"
     uid: "4"
     annotations:
-      link.argocd.argoproj.io/external-link: http://my-grafana.com/pre-generated-link
+      link.argocd.argoproj.io/external-link: http://my-grafana.example.com/pre-generated-link
   spec:
     selector:
       app: guestbook
@@ -74,7 +75,7 @@ var (
       serviceName: not-found-service
       servicePort: 443
     rules:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
       http:
         paths:
         - backend:
@@ -86,7 +87,7 @@ var (
             servicePort: https
           path: /
     tls:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
     secretName: my-tls-secret
   status:
     loadBalancer:
@@ -101,13 +102,13 @@ var (
     namespace: default
     uid: "4"
     annotations:
-      link.argocd.argoproj.io/external-link: http://my-grafana.com/ingress-link
+      link.argocd.argoproj.io/external-link: http://my-grafana.example.com/ingress-link
   spec:
     backend:
       serviceName: not-found-service
       servicePort: 443
     rules:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
       http:
         paths:
         - backend:
@@ -119,7 +120,7 @@ var (
             servicePort: https
           path: /
     tls:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
     secretName: my-tls-secret
   status:
     loadBalancer:
@@ -138,7 +139,7 @@ var (
       serviceName: not-found-service
       servicePort: 443
     rules:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
       http:
         paths:
         - backend:
@@ -150,7 +151,7 @@ var (
             servicePort: https
           path: /*
     tls:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
     secretName: my-tls-secret
   status:
     loadBalancer:
@@ -169,7 +170,7 @@ var (
       serviceName: not-found-service
       servicePort: 443
     rules:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
       http:
         paths:
         - backend:
@@ -199,7 +200,7 @@ var (
         port:
           number: 443
     rules:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
       http:
         paths:
         - backend:
@@ -215,7 +216,7 @@ var (
                 name: https
           path: /
     tls:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
     secretName: my-tls-secret
   status:
     loadBalancer:
@@ -313,7 +314,7 @@ status:
 func TestGetServiceInfo(t *testing.T) {
 	info := &ResourceInfo{}
 	populateNodeInfo(testService, info, []string{})
-	assert.Equal(t, 0, len(info.Info))
+	assert.Empty(t, info.Info)
 	assert.Equal(t, &v1alpha1.ResourceNetworkingInfo{
 		TargetLabels: map[string]string{"app": "guestbook"},
 		Ingress:      []v1.LoadBalancerIngress{{Hostname: "localhost"}},
@@ -323,18 +324,18 @@ func TestGetServiceInfo(t *testing.T) {
 func TestGetLinkAnnotatedServiceInfo(t *testing.T) {
 	info := &ResourceInfo{}
 	populateNodeInfo(testLinkAnnotatedService, info, []string{})
-	assert.Equal(t, 0, len(info.Info))
+	assert.Empty(t, info.Info)
 	assert.Equal(t, &v1alpha1.ResourceNetworkingInfo{
 		TargetLabels: map[string]string{"app": "guestbook"},
 		Ingress:      []v1.LoadBalancerIngress{{Hostname: "localhost"}},
-		ExternalURLs: []string{"http://my-grafana.com/pre-generated-link"},
+		ExternalURLs: []string{"http://my-grafana.example.com/pre-generated-link"},
 	}, info.NetworkingInfo)
 }
 
 func TestGetIstioVirtualServiceInfo(t *testing.T) {
 	info := &ResourceInfo{}
 	populateNodeInfo(testIstioVirtualService, info, []string{})
-	assert.Equal(t, 0, len(info.Info))
+	assert.Empty(t, info.Info)
 	require.NotNil(t, info.NetworkingInfo)
 	require.NotNil(t, info.NetworkingInfo.TargetRefs)
 	assert.Contains(t, info.NetworkingInfo.TargetRefs, v1alpha1.ResourceRef{
@@ -355,7 +356,7 @@ func TestGetIstioVirtualServiceInfo(t *testing.T) {
 }
 
 func TestGetIngressInfo(t *testing.T) {
-	var tests = []struct {
+	tests := []struct {
 		Ingress *unstructured.Unstructured
 	}{
 		{testIngress},
@@ -364,7 +365,7 @@ func TestGetIngressInfo(t *testing.T) {
 	for _, tc := range tests {
 		info := &ResourceInfo{}
 		populateNodeInfo(tc.Ingress, info, []string{})
-		assert.Equal(t, 0, len(info.Info))
+		assert.Empty(t, info.Info)
 		sort.Slice(info.NetworkingInfo.TargetRefs, func(i, j int) bool {
 			return strings.Compare(info.NetworkingInfo.TargetRefs[j].Name, info.NetworkingInfo.TargetRefs[i].Name) < 0
 		})
@@ -381,7 +382,7 @@ func TestGetIngressInfo(t *testing.T) {
 				Kind:      kube.ServiceKind,
 				Name:      "helm-guestbook",
 			}},
-			ExternalURLs: []string{"https://helm-guestbook.com/"},
+			ExternalURLs: []string{"https://helm-guestbook.example.com/"},
 		}, info.NetworkingInfo)
 	}
 }
@@ -389,7 +390,7 @@ func TestGetIngressInfo(t *testing.T) {
 func TestGetLinkAnnotatedIngressInfo(t *testing.T) {
 	info := &ResourceInfo{}
 	populateNodeInfo(testLinkAnnotatedIngress, info, []string{})
-	assert.Equal(t, 0, len(info.Info))
+	assert.Empty(t, info.Info)
 	sort.Slice(info.NetworkingInfo.TargetRefs, func(i, j int) bool {
 		return strings.Compare(info.NetworkingInfo.TargetRefs[j].Name, info.NetworkingInfo.TargetRefs[i].Name) < 0
 	})
@@ -406,14 +407,14 @@ func TestGetLinkAnnotatedIngressInfo(t *testing.T) {
 			Kind:      kube.ServiceKind,
 			Name:      "helm-guestbook",
 		}},
-		ExternalURLs: []string{"https://helm-guestbook.com/", "http://my-grafana.com/ingress-link"},
+		ExternalURLs: []string{"http://my-grafana.example.com/ingress-link", "https://helm-guestbook.example.com/"},
 	}, info.NetworkingInfo)
 }
 
 func TestGetIngressInfoWildCardPath(t *testing.T) {
 	info := &ResourceInfo{}
 	populateNodeInfo(testIngressWildCardPath, info, []string{})
-	assert.Equal(t, 0, len(info.Info))
+	assert.Empty(t, info.Info)
 	sort.Slice(info.NetworkingInfo.TargetRefs, func(i, j int) bool {
 		return strings.Compare(info.NetworkingInfo.TargetRefs[j].Name, info.NetworkingInfo.TargetRefs[i].Name) < 0
 	})
@@ -430,14 +431,14 @@ func TestGetIngressInfoWildCardPath(t *testing.T) {
 			Kind:      kube.ServiceKind,
 			Name:      "helm-guestbook",
 		}},
-		ExternalURLs: []string{"https://helm-guestbook.com/"},
+		ExternalURLs: []string{"https://helm-guestbook.example.com/"},
 	}, info.NetworkingInfo)
 }
 
 func TestGetIngressInfoWithoutTls(t *testing.T) {
 	info := &ResourceInfo{}
 	populateNodeInfo(testIngressWithoutTls, info, []string{})
-	assert.Equal(t, 0, len(info.Info))
+	assert.Empty(t, info.Info)
 	sort.Slice(info.NetworkingInfo.TargetRefs, func(i, j int) bool {
 		return strings.Compare(info.NetworkingInfo.TargetRefs[j].Name, info.NetworkingInfo.TargetRefs[i].Name) < 0
 	})
@@ -454,7 +455,7 @@ func TestGetIngressInfoWithoutTls(t *testing.T) {
 			Kind:      kube.ServiceKind,
 			Name:      "helm-guestbook",
 		}},
-		ExternalURLs: []string{"http://helm-guestbook.com/"},
+		ExternalURLs: []string{"http://helm-guestbook.example.com/"},
 	}, info.NetworkingInfo)
 }
 
@@ -494,6 +495,7 @@ func TestGetIngressInfoWithHost(t *testing.T) {
 		ExternalURLs: []string{"https://107.178.210.11/"},
 	}, info.NetworkingInfo)
 }
+
 func TestGetIngressInfoNoHost(t *testing.T) {
 	ingress := strToUnstructured(`
   apiVersion: extensions/v1beta1
@@ -524,8 +526,9 @@ func TestGetIngressInfoNoHost(t *testing.T) {
 			Name:      "helm-guestbook",
 		}},
 	}, info.NetworkingInfo)
-	assert.Equal(t, len(info.NetworkingInfo.ExternalURLs), 0)
+	assert.Empty(t, info.NetworkingInfo.ExternalURLs)
 }
+
 func TestExternalUrlWithSubPath(t *testing.T) {
 	ingress := strToUnstructured(`
   apiVersion: networking.k8s.io/v1
@@ -554,6 +557,7 @@ func TestExternalUrlWithSubPath(t *testing.T) {
 	expectedExternalUrls := []string{"https://107.178.210.11/my/sub/path/"}
 	assert.Equal(t, expectedExternalUrls, info.NetworkingInfo.ExternalURLs)
 }
+
 func TestExternalUrlWithMultipleSubPaths(t *testing.T) {
 	ingress := strToUnstructured(`
   apiVersion: networking.k8s.io/v1
@@ -563,7 +567,7 @@ func TestExternalUrlWithMultipleSubPaths(t *testing.T) {
     namespace: default
   spec:
     rules:
-    - host: helm-guestbook.com
+    - host: helm-guestbook.example.com
       http:
         paths:
         - backend:
@@ -587,12 +591,13 @@ func TestExternalUrlWithMultipleSubPaths(t *testing.T) {
 	info := &ResourceInfo{}
 	populateNodeInfo(ingress, info, []string{})
 
-	expectedExternalUrls := []string{"https://helm-guestbook.com/my/sub/path/", "https://helm-guestbook.com/my/sub/path/2", "https://helm-guestbook.com"}
+	expectedExternalUrls := []string{"https://helm-guestbook.example.com/my/sub/path/", "https://helm-guestbook.example.com/my/sub/path/2", "https://helm-guestbook.example.com"}
 	actualURLs := info.NetworkingInfo.ExternalURLs
 	sort.Strings(expectedExternalUrls)
 	sort.Strings(actualURLs)
 	assert.Equal(t, expectedExternalUrls, actualURLs)
 }
+
 func TestExternalUrlWithNoSubPath(t *testing.T) {
 	ingress := strToUnstructured(`
   apiVersion: networking.k8s.io/v1
@@ -659,7 +664,7 @@ func TestCustomLabel(t *testing.T) {
 	info := &ResourceInfo{}
 	populateNodeInfo(configmap, info, []string{"my-label"})
 
-	assert.Equal(t, 0, len(info.Info))
+	assert.Empty(t, info.Info)
 
 	configmap = strToUnstructured(`
   apiVersion: v1
@@ -672,7 +677,7 @@ func TestCustomLabel(t *testing.T) {
 	info = &ResourceInfo{}
 	populateNodeInfo(configmap, info, []string{"my-label", "other-label"})
 
-	assert.Equal(t, 1, len(info.Info))
+	assert.Len(t, info.Info, 1)
 	assert.Equal(t, "my-label", info.Info[0].Name)
 	assert.Equal(t, "value", info.Info[0].Value)
 
@@ -688,9 +693,68 @@ func TestCustomLabel(t *testing.T) {
 	info = &ResourceInfo{}
 	populateNodeInfo(configmap, info, []string{"my-label", "other-label"})
 
-	assert.Equal(t, 2, len(info.Info))
+	assert.Len(t, info.Info, 2)
 	assert.Equal(t, "my-label", info.Info[0].Name)
 	assert.Equal(t, "value", info.Info[0].Value)
 	assert.Equal(t, "other-label", info.Info[1].Name)
 	assert.Equal(t, "value2", info.Info[1].Value)
 }
+
+func TestManifestHash(t *testing.T) {
+	manifest := strToUnstructured(`
+  apiVersion: v1
+  kind: Pod
+  metadata:
+    name: helm-guestbook-pod
+    namespace: default
+    ownerReferences:
+    - apiVersion: extensions/v1beta1
+      kind: ReplicaSet
+      name: helm-guestbook-rs
+    resourceVersion: "123"
+    labels:
+      app: guestbook
+  spec:
+    nodeName: minikube
+    containers:
+    - image: bar
+      resources:
+        requests:
+          memory: 128Mi
+`)
+
+	ignores := []v1alpha1.ResourceIgnoreDifferences{
+		{
+			Group:        "*",
+			Kind:         "*",
+			JSONPointers: []string{"/metadata/resourceVersion"},
+		},
+	}
+
+	data, _ := strToUnstructured(`
+  apiVersion: v1
+  kind: Pod
+  metadata:
+    name: helm-guestbook-pod
+    namespace: default
+    ownerReferences:
+    - apiVersion: extensions/v1beta1
+      kind: ReplicaSet
+      name: helm-guestbook-rs
+    labels:
+      app: guestbook
+  spec:
+    nodeName: minikube
+    containers:
+    - image: bar
+      resources:
+        requests:
+          memory: 128Mi
+`).MarshalJSON()
+
+	expected := hash(data)
+
+	hash, err := generateManifestHash(manifest, ignores, nil, normalizers.IgnoreNormalizerOpts{})
+	assert.Equal(t, expected, hash)
+	assert.NoError(t, err)
+}
