@@ -1,36 +1,39 @@
-import {DataLoader, DropDown, DropDownMenu, MenuItem, NotificationType, Tooltip} from 'argo-ui';
+import {DataLoader, DropDown, DropDownMenu, MenuItem, Tooltip} from 'argo-ui';
 import * as PropTypes from 'prop-types';
 import * as React from 'react';
 import Moment from 'react-moment';
 
 import {AppContext} from '../../../shared/context';
-import {CheckboxField, EmptyState, ErrorNotification} from '../../../shared/components';
+import {EmptyState} from '../../../shared/components';
 import {Application, ApplicationTree, HostResourceInfo, InfoItem, Node, Pod, ResourceName, ResourceNode, ResourceStatus} from '../../../shared/models';
 import {PodViewPreferences, services, ViewPreferences} from '../../../shared/services';
 
 import {ResourceTreeNode} from '../application-resource-tree/application-resource-tree';
 import {ResourceIcon} from '../resource-icon';
 import {ResourceLabel} from '../resource-label';
-import {ComparisonStatusIcon, HealthStatusIcon, nodeKey, PodHealthIcon} from '../utils';
+import {ComparisonStatusIcon, isYoungerThanXMinutes, HealthStatusIcon, nodeKey, PodHealthIcon} from '../utils';
 
 import './pod-view.scss';
+import {PodTooltip} from './pod-tooltip';
 
 interface PodViewProps {
     tree: ApplicationTree;
     onItemClick: (fullName: string) => void;
     app: Application;
     nodeMenu?: (node: ResourceNode) => React.ReactNode;
+    quickStarts?: (node: ResourceNode) => React.ReactNode;
 }
 
 export type PodGroupType = 'topLevelResource' | 'parentResource' | 'node';
 
-interface PodGroup extends Partial<ResourceNode> {
+export interface PodGroup extends Partial<ResourceNode> {
     type: PodGroupType;
     pods: Pod[];
     info?: InfoItem[];
     hostResourcesInfo?: HostResourceInfo[];
     resourceStatus?: Partial<ResourceStatus>;
     renderMenu?: () => React.ReactNode;
+    renderQuickStarts?: () => React.ReactNode;
     fullName?: string;
 }
 
@@ -49,6 +52,7 @@ export class PodView extends React.Component<PodViewProps> {
                 {prefs => {
                     const podPrefs = prefs.appDetails.podView || ({} as PodViewPreferences);
                     const groups = this.processTree(podPrefs.sortMode, this.props.tree.hosts || []) || [];
+
                     return (
                         <React.Fragment>
                             <div className='pod-view__settings'>
@@ -104,7 +108,9 @@ export class PodView extends React.Component<PodViewProps> {
                                                                 <div>
                                                                     {group.resourceStatus.health && <HealthStatusIcon state={group.resourceStatus.health} />}
                                                                     &nbsp;
-                                                                    {group.resourceStatus.status && <ComparisonStatusIcon status={group.resourceStatus.status} />}
+                                                                    {group.resourceStatus.status && (
+                                                                        <ComparisonStatusIcon status={group.resourceStatus.status} resource={group.resourceStatus} />
+                                                                    )}
                                                                 </div>
                                                             )}
                                                         </div>
@@ -139,9 +145,7 @@ export class PodView extends React.Component<PodViewProps> {
                                                                     </Moment>
                                                                 </div>
                                                             ) : null}
-                                                            {group.info?.map(infoItem => (
-                                                                <div key={infoItem.name}>{infoItem.value}</div>
-                                                            ))}
+                                                            {group.info?.map(infoItem => <div key={infoItem.name}>{infoItem.value}</div>)}
                                                         </div>
                                                     )}
                                                 </div>
@@ -153,99 +157,48 @@ export class PodView extends React.Component<PodViewProps> {
                                                     )}
                                                     <div className='pod-view__node__pod-container pod-view__node__container'>
                                                         <div className='pod-view__node__pod-container__pods'>
-                                                            {group.pods.map(pod => (
-                                                                <DropDownMenu
-                                                                    key={pod.uid}
-                                                                    anchor={() => (
-                                                                        <Tooltip
-                                                                            content={
-                                                                                <div>
-                                                                                    {pod.metadata.name}
-                                                                                    <div>Health: {pod.health}</div>
-                                                                                </div>
-                                                                            }
-                                                                            popperOptions={{
-                                                                                modifiers: {
-                                                                                    preventOverflow: {
-                                                                                        enabled: true
-                                                                                    },
-                                                                                    hide: {
-                                                                                        enabled: false
-                                                                                    },
-                                                                                    flip: {
-                                                                                        enabled: false
-                                                                                    }
-                                                                                }
-                                                                            }}
-                                                                            key={pod.metadata.name}>
-                                                                            <div className={`pod-view__node__pod pod-view__node__pod--${pod.health.toLowerCase()}`}>
-                                                                                <PodHealthIcon state={{status: pod.health, message: ''}} />
-                                                                            </div>
-                                                                        </Tooltip>
-                                                                    )}
-                                                                    items={[
-                                                                        {
-                                                                            title: (
-                                                                                <React.Fragment>
-                                                                                    <i className='fa fa-info-circle' /> Info
-                                                                                </React.Fragment>
-                                                                            ),
-                                                                            action: () => this.props.onItemClick(pod.fullName)
-                                                                        },
-                                                                        {
-                                                                            title: (
-                                                                                <React.Fragment>
-                                                                                    <i className='fa fa-align-left' /> Logs
-                                                                                </React.Fragment>
-                                                                            ),
-                                                                            action: () => {
-                                                                                this.appContext.apis.navigation.goto('.', {node: pod.fullName, tab: 'logs'}, {replace: true});
-                                                                            }
-                                                                        },
-                                                                        {
-                                                                            title: (
-                                                                                <React.Fragment>
-                                                                                    <i className='fa fa-trash' /> Delete
-                                                                                </React.Fragment>
-                                                                            ),
-                                                                            action: async () => {
-                                                                                this.appContext.apis.popup.prompt(
-                                                                                    'Delete pod',
-                                                                                    () => (
-                                                                                        <div>
-                                                                                            <p>Are your sure you want to delete Pod '{pod.name}'?</p>
-                                                                                            <div className='argo-form-row' style={{paddingLeft: '30px'}}>
-                                                                                                <CheckboxField id='force-delete-checkbox' field='force' />
-                                                                                                <label htmlFor='force-delete-checkbox'>Force delete</label>
-                                                                                            </div>
-                                                                                        </div>
-                                                                                    ),
-                                                                                    {
-                                                                                        submit: async (vals, _, close) => {
-                                                                                            try {
-                                                                                                await services.applications.deleteResource(
-                                                                                                    this.props.app.metadata.name,
-                                                                                                    pod,
-                                                                                                    !!vals.force,
-                                                                                                    false
-                                                                                                );
-                                                                                                close();
-                                                                                            } catch (e) {
-                                                                                                this.appContext.apis.notifications.show({
-                                                                                                    content: <ErrorNotification title='Unable to delete resource' e={e} />,
-                                                                                                    type: NotificationType.Error
-                                                                                                });
+                                                            {group.pods.map(
+                                                                pod =>
+                                                                    this.props.nodeMenu && (
+                                                                        <DropDown
+                                                                            key={pod.uid}
+                                                                            isMenu={true}
+                                                                            anchor={() => (
+                                                                                <Tooltip
+                                                                                    content={<PodTooltip pod={pod} />}
+                                                                                    popperOptions={{
+                                                                                        modifiers: {
+                                                                                            preventOverflow: {
+                                                                                                enabled: true
+                                                                                            },
+                                                                                            hide: {
+                                                                                                enabled: false
+                                                                                            },
+                                                                                            flip: {
+                                                                                                enabled: false
                                                                                             }
                                                                                         }
-                                                                                    }
-                                                                                );
-                                                                            }
-                                                                        }
-                                                                    ]}
-                                                                />
-                                                            ))}
+                                                                                    }}
+                                                                                    key={pod.metadata.name}>
+                                                                                    <div style={{position: 'relative'}}>
+                                                                                        {isYoungerThanXMinutes(pod, 30) && (
+                                                                                            <i className='fas fa-circle pod-view__node__pod pod-view__node__pod__new-pod-icon' />
+                                                                                        )}
+                                                                                        <div className={`pod-view__node__pod pod-view__node__pod--${pod.health.toLowerCase()}`}>
+                                                                                            <PodHealthIcon state={{status: pod.health, message: ''}} />
+                                                                                        </div>
+                                                                                    </div>
+                                                                                </Tooltip>
+                                                                            )}>
+                                                                            {() => this.props.nodeMenu(pod)}
+                                                                        </DropDown>
+                                                                    )
+                                                            )}
                                                         </div>
                                                         <div className='pod-view__node__label'>PODS</div>
+                                                        {(podPrefs.sortMode === 'parentResource' || podPrefs.sortMode === 'topLevelResource') && (
+                                                            <div key={group.uid}>{group.renderQuickStarts()}</div>
+                                                        )}
                                                     </div>
                                                 </div>
                                             </div>
@@ -274,6 +227,7 @@ export class PodView extends React.Component<PodViewProps> {
                 </React.Fragment>
             ),
             action: () => {
+                this.appContext.apis.navigation.goto('.', {podSortMode: mode});
                 services.viewPreferences.updatePreferences({appDetails: {...prefs.appDetails, podView: {...podPrefs, sortMode: mode}}});
             }
         }));
@@ -307,6 +261,7 @@ export class PodView extends React.Component<PodViewProps> {
 
         const statusByKey = new Map<string, ResourceStatus>();
         this.props.app.status?.resources?.forEach(res => statusByKey.set(nodeKey(res), res));
+
         (tree.nodes || []).forEach((rnode: ResourceTreeNode) => {
             // make sure each node has not null/undefined parentRefs field
             rnode.parentRefs = rnode.parentRefs || [];
@@ -315,6 +270,7 @@ export class PodView extends React.Component<PodViewProps> {
                 parentsFor[rnode.uid] = rnode.parentRefs as PodGroup[];
                 const fullName = nodeKey(rnode);
                 const status = statusByKey.get(fullName);
+
                 if ((rnode.parentRefs || []).length === 0) {
                     rnode.root = rnode;
                 }
@@ -325,8 +281,9 @@ export class PodView extends React.Component<PodViewProps> {
                     ...rnode,
                     info: (rnode.info || []).filter(i => !i.name.includes('Resource.')),
                     createdAt: rnode.createdAt,
-                    resourceStatus: {health: rnode.health, status: status ? status.status : null},
-                    renderMenu: () => this.props.nodeMenu(rnode)
+                    resourceStatus: {health: rnode.health, status: status ? status.status : null, requiresPruning: status && status.requiresPruning ? true : false},
+                    renderMenu: () => this.props.nodeMenu(rnode),
+                    renderQuickStarts: () => this.props.quickStarts(rnode)
                 };
             }
         });
@@ -411,7 +368,7 @@ const labelForSortMode = {
     topLevelResource: 'Top Level Resource'
 };
 
-const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
+const sizes = ['Bytes', 'Ki', 'Mi', 'Gi', 'Ti', 'Pi', 'Ei', 'Zi', 'Yi'];
 function formatSize(bytes: number) {
     if (!bytes) {
         return '0 Bytes';
