@@ -1,8 +1,9 @@
-import {DropDown, DropDownMenu, Tooltip} from 'argo-ui';
+import {DropDown, Tooltip} from 'argo-ui';
 import * as classNames from 'classnames';
 import * as dagre from 'dagre';
 import * as React from 'react';
 import Moment from 'react-moment';
+import * as moment from 'moment';
 
 import * as models from '../../../shared/models';
 
@@ -14,20 +15,19 @@ import {ResourceLabel} from '../resource-label';
 import {
     BASE_COLORS,
     ComparisonStatusIcon,
-    deletePodAction,
     getAppOverridesCount,
-    getExternalUrls,
     HealthStatusIcon,
     isAppNode,
     isYoungerThanXMinutes,
     NodeId,
     nodeKey,
-    PodHealthIcon
+    PodHealthIcon,
+    getUsrMsgKeyToDisplay
 } from '../utils';
 import {NodeUpdateAnimation} from './node-update-animation';
 import {PodGroup} from '../application-pod-view/pod-view';
-import {ArrowConnector} from './arrow-connector';
 import './application-resource-tree.scss';
+import {ArrowConnector} from './arrow-connector';
 
 function treeNodeKey(node: NodeId & {uid?: string}) {
     return node.uid || nodeKey(node);
@@ -59,7 +59,11 @@ export interface ApplicationResourceTreeProps {
     appContext?: AppContext;
     showOrphanedResources: boolean;
     showCompactNodes: boolean;
+    userMsgs: models.UserMessages[];
+    updateUsrHelpTipMsgs: (userMsgs: models.UserMessages) => void;
+    setShowCompactNodes: (showCompactNodes: boolean) => void;
     zoom: number;
+    podGroupCount: number;
     filters?: string[];
     setTreeFilterGraph?: (filterGraph: any[]) => void;
     nameDirection: boolean;
@@ -88,17 +92,8 @@ const NODE_TYPES = {
     groupedNodes: 'grouped_nodes',
     podGroup: 'pod_group'
 };
-
 // generate lots of colors with different darkness
-const TRAFFIC_COLORS = [0, 0.25, 0.4, 0.6]
-    .map(darken =>
-        BASE_COLORS.map(item =>
-            color(item)
-                .darken(darken)
-                .hex()
-        )
-    )
-    .reduce((first, second) => first.concat(second), []);
+const TRAFFIC_COLORS = [0, 0.25, 0.4, 0.6].map(darken => BASE_COLORS.map(item => color(item).darken(darken).hex())).reduce((first, second) => first.concat(second), []);
 
 function getGraphSize(nodes: dagre.Node[]): {width: number; height: number} {
     let width = 0;
@@ -110,7 +105,7 @@ function getGraphSize(nodes: dagre.Node[]): {width: number; height: number} {
     return {width, height};
 }
 
-function groupNodes(nodes: any[], graph: dagre.graphlib.Graph) {
+function groupNodes(nodes: ResourceTreeNode[], graph: dagre.graphlib.Graph) {
     function getNodeGroupingInfo(nodeId: string) {
         const node = graph.node(nodeId);
         return {
@@ -174,22 +169,34 @@ function groupNodes(nodes: any[], graph: dagre.graphlib.Graph) {
         groupedNodesArr.forEach((obj: {kind: string; nodeIds: string[]; parentIds: dagre.Node[]}) => {
             const {nodeIds, kind, parentIds} = obj;
             const groupedNodeIds: string[] = [];
+            const podGroupIds: string[] = [];
             nodeIds.forEach((nodeId: string) => {
                 const index = nodes.findIndex(node => nodeId === node.uid || nodeId === nodeKey(node));
-                if (index > -1) {
+                const graphNode = graph.node(nodeId);
+                if (!graphNode?.podGroup && index > -1) {
                     groupedNodeIds.push(nodeId);
+                } else {
+                    podGroupIds.push(nodeId);
                 }
-                graph.removeNode(nodeId);
             });
-            graph.setNode(`${parentIds[0].toString()}/child/${kind}`, {
-                kind,
-                groupedNodeIds,
-                height: NODE_HEIGHT,
-                width: NODE_WIDTH,
-                count: nodeIds.length,
-                type: NODE_TYPES.groupedNodes
-            });
-            graph.setEdge(parentIds[0].toString(), `${parentIds[0].toString()}/child/${kind}`);
+            const reducedNodeIds = nodeIds.reduce((acc, aNodeId) => {
+                if (podGroupIds.findIndex(i => i === aNodeId) < 0) {
+                    acc.push(aNodeId);
+                }
+                return acc;
+            }, []);
+            if (groupedNodeIds.length > 1) {
+                groupedNodeIds.forEach(n => graph.removeNode(n));
+                graph.setNode(`${parentIds[0].toString()}/child/${kind}`, {
+                    kind,
+                    groupedNodeIds,
+                    height: NODE_HEIGHT,
+                    width: NODE_WIDTH,
+                    count: reducedNodeIds.length,
+                    type: NODE_TYPES.groupedNodes
+                });
+                graph.setEdge(parentIds[0].toString(), `${parentIds[0].toString()}/child/${kind}`);
+            }
         });
     }
 }
@@ -217,6 +224,14 @@ export function compareNodes(first: ResourceTreeNode, second: ResourceTreeNode)
         }
         return value.replace(/^Rev:/, '');
     }
+    if (first.kind === 'ReplicaSet') {
+        return (
+            orphanedToInt(first.orphaned) - orphanedToInt(second.orphaned) ||
+            compareRevision(getRevision(second), getRevision(first)) ||
+            nodeKey(first).localeCompare(nodeKey(second)) ||
+            0
+        );
+    }
     return (
         orphanedToInt(first.orphaned) - orphanedToInt(second.orphaned) ||
         nodeKey(first).localeCompare(nodeKey(second)) ||
@@ -272,10 +287,22 @@ function renderGroupedNodes(props: ApplicationResourceTreeProps, node: {count: n
                     <br />
                     <div className='application-resource-tree__node-kind'>{ResourceLabel({kind: node.kind})}</div>
                 </div>
-                <div className='application-resource-tree__node-content-wrap-overflow'>
-                    <a className='application-resource-tree__node-title' onClick={() => props.onGroupdNodeClick && props.onGroupdNodeClick(node.groupedNodeIds)}>
-                        click to show details of {node.count} collapsed {node.kind}
-                    </a>
+                <div
+                    className='application-resource-tree__node-title application-resource-tree__direction-center-left'
+                    onClick={() => props.onGroupdNodeClick && props.onGroupdNodeClick(node.groupedNodeIds)}
+                    title={`Click to see details of ${node.count} collapsed ${node.kind} and doesn't contains any active pods`}>
+                    {node.count} {node.kind}s
+                    <span style={{paddingLeft: '.5em', fontSize: 'small'}}>
+                        {node.kind === 'ReplicaSet' ? (
+                            <i
+                                className='fa-solid fa-cart-flatbed icon-background'
+                                title={`Click to see details of ${node.count} collapsed ${node.kind} and doesn't contains any active pods`}
+                                key={node.uid}
+                            />
+                        ) : (
+                            <i className='fa fa-info-circle icon-background' title={`Click to see details of ${node.count} collapsed ${node.kind}`} key={node.uid} />
+                        )}
+                    </span>
                 </div>
             </div>
             {indicators.map(i => (
@@ -376,10 +403,7 @@ function renderPodGroup(props: ApplicationResourceTreeProps, id: string, node: R
     }
     const appNode = isAppNode(node);
     const rootNode = !node.root;
-    let extLinks: string[] = props.app.status.summary.externalURLs;
-    if (rootNode) {
-        extLinks = getExternalUrls(props.app.metadata.annotations, props.app.status.summary.externalURLs);
-    }
+    const extLinks: string[] = props.app.status.summary.externalURLs;
     const podGroupChildren = childMap.get(treeNodeKey(node));
     const nonPodChildren = podGroupChildren?.reduce((acc, child) => {
         if (child.kind !== 'Pod') {
@@ -391,21 +415,47 @@ function renderPodGroup(props: ApplicationResourceTreeProps, id: string, node: R
     const margin = 8;
     let topExtra = 0;
     const podGroup = node.podGroup;
+    const podGroupHealthy = [];
+    const podGroupDegraded = [];
+    const podGroupInProgress = [];
+
+    for (const pod of podGroup?.pods || []) {
+        switch (pod.health) {
+            case 'Healthy':
+                podGroupHealthy.push(pod);
+                break;
+            case 'Degraded':
+                podGroupDegraded.push(pod);
+                break;
+            case 'Progressing':
+                podGroupInProgress.push(pod);
+        }
+    }
+
+    const showPodGroupByStatus = props.tree.nodes.filter((rNode: ResourceTreeNode) => rNode.kind === 'Pod').length >= props.podGroupCount;
+    const numberOfRows = showPodGroupByStatus
+        ? [podGroupHealthy, podGroupDegraded, podGroupInProgress].reduce((total, podGroupByStatus) => total + (podGroupByStatus.filter(pod => pod).length > 0 ? 1 : 0), 0)
+        : Math.ceil(podGroup?.pods.length / 8);
+
     if (podGroup) {
-        const numberOfRows = Math.ceil(podGroup.pods.length / 8);
         topExtra = margin + (POD_NODE_HEIGHT / 2 + 30 * numberOfRows) / 2;
     }
+
     return (
         <div
-            onClick={() => props.onNodeClick && props.onNodeClick(fullName)}
             className={classNames('application-resource-tree__node', {
                 'active': fullName === props.selectedNodeFullName,
                 'application-resource-tree__node--orphaned': node.orphaned
             })}
             title={describeNode(node)}
-            style={{left: node.x, top: node.y - topExtra, width: node.width, height: node.height}}>
+            style={{
+                left: node.x,
+                top: node.y - topExtra,
+                width: node.width,
+                height: showPodGroupByStatus ? POD_NODE_HEIGHT + 20 * numberOfRows : node.height
+            }}>
             <NodeUpdateAnimation resourceVersion={node.resourceVersion} />
-            <div className={`application-resource-tree__node__top-part`}>
+            <div onClick={() => props.onNodeClick && props.onNodeClick(fullName)} className={`application-resource-tree__node__top-part`}>
                 <div
                     className={classNames('application-resource-tree__node-kind-icon', {
                         'application-resource-tree__node-kind-icon--big': rootNode
@@ -419,7 +469,8 @@ function renderPodGroup(props: ApplicationResourceTreeProps, id: string, node: R
                         className={classNames('application-resource-tree__node-title', {
                             'application-resource-tree__direction-right': props.nameDirection,
                             'application-resource-tree__direction-left': !props.nameDirection
-                        })}>
+                        })}
+                        onClick={() => props.onGroupdNodeClick && props.onGroupdNodeClick(node.groupedNodeIds)}>
                         {node.name}
                     </span>
                     <span
@@ -471,11 +522,15 @@ function renderPodGroup(props: ApplicationResourceTreeProps, id: string, node: R
                         ))}
                     {(node.info || []).length > 4 && (
                         <Tooltip
-                            content={(node.info || []).map(i => (
-                                <div key={i.name}>
-                                    {i.name}: {i.value}
-                                </div>
-                            ))}
+                            content={
+                                <>
+                                    {(node.info || []).map(i => (
+                                        <div key={i.name}>
+                                            {i.name}: {i.value}
+                                        </div>
+                                    ))}
+                                </>
+                            }
                             key={node.uid}>
                             <span className='application-resource-tree__node-label' title='More'>
                                 More
@@ -486,6 +541,7 @@ function renderPodGroup(props: ApplicationResourceTreeProps, id: string, node: R
                 {props.nodeMenu && (
                     <div className='application-resource-tree__node-menu'>
                         <DropDown
+                            key={node.uid}
                             isMenu={true}
                             anchor={() => (
                                 <button className='argo-button argo-button--light argo-button--lg argo-button--short'>
@@ -498,93 +554,96 @@ function renderPodGroup(props: ApplicationResourceTreeProps, id: string, node: R
                 )}
             </div>
             <div className='application-resource-tree__node--lower-section'>
-                {podGroup && (
-                    <div className={`application-resource-tree__node--lower-section__pod-group`} key={podGroup.fullName || podGroup.name}>
-                        <div className='application-resource-tree__node--lower-section__pod-group__pod-container'>
-                            <div className='application-resource-tree__node--lower-section__pod-group__pod-container__pods'>
-                                {podGroup.pods.map(pod => (
-                                    <DropDownMenu
-                                        key={pod.uid}
-                                        anchor={() => (
-                                            <Tooltip
-                                                content={
-                                                    <div>
-                                                        {pod.metadata.name}
-                                                        <div>Health: {pod.health}</div>
-                                                        {pod.createdAt && (
-                                                            <span>
-                                                                <span>Created: </span>
-                                                                <Moment fromNow={true} ago={true}>
-                                                                    {pod.createdAt}
-                                                                </Moment>
-                                                                <span> ago ({<Moment local={true}>{pod.createdAt}</Moment>})</span>
-                                                            </span>
-                                                        )}
-                                                    </div>
-                                                }
-                                                popperOptions={{
-                                                    modifiers: {
-                                                        preventOverflow: {
-                                                            enabled: true
-                                                        },
-                                                        hide: {
-                                                            enabled: false
-                                                        },
-                                                        flip: {
-                                                            enabled: false
-                                                        }
-                                                    }
-                                                }}
-                                                key={pod.metadata.name}>
-                                                <div style={{position: 'relative'}}>
-                                                    {isYoungerThanXMinutes(pod, 30) && (
-                                                        <i className='fas fa-star application-resource-tree__node--lower-section__pod-group__pod application-resource-tree__node--lower-section__pod-group__pod__star-icon' />
-                                                    )}
-                                                    <div
-                                                        className={`application-resource-tree__node--lower-section__pod-group__pod application-resource-tree__node--lower-section__pod-group__pod--${pod.health.toLowerCase()}`}>
-                                                        <PodHealthIcon state={{status: pod.health, message: ''}} />
-                                                    </div>
-                                                </div>
-                                            </Tooltip>
-                                        )}
-                                        items={[
-                                            {
-                                                title: (
-                                                    <React.Fragment>
-                                                        <i className='fa fa-info-circle' /> Info
-                                                    </React.Fragment>
-                                                ),
-                                                action: () => props.onNodeClick(pod.fullName)
-                                            },
-                                            {
-                                                title: (
-                                                    <React.Fragment>
-                                                        <i className='fa fa-align-left' /> Logs
-                                                    </React.Fragment>
-                                                ),
-                                                action: () => {
-                                                    props.appContext.apis.navigation.goto('.', {node: pod.fullName, tab: 'logs'}, {replace: true});
-                                                }
-                                            },
-                                            {
-                                                title: (
-                                                    <React.Fragment>
-                                                        <i className='fa fa-times-circle' /> Delete
-                                                    </React.Fragment>
-                                                ),
-                                                action: () => {
-                                                    deletePodAction(pod, props.appContext, props.app.metadata.name, props.app.metadata.namespace);
+                {[podGroupHealthy, podGroupDegraded, podGroupInProgress].map((pods, index) => {
+                    if (pods.length > 0) {
+                        return (
+                            <div key={index} className={`application-resource-tree__node--lower-section__pod-group`}>
+                                {renderPodGroupByStatus(props, node, pods, showPodGroupByStatus)}
+                            </div>
+                        );
+                    }
+                })}
+            </div>
+        </div>
+    );
+}
+
+function renderPodGroupByStatus(props: ApplicationResourceTreeProps, node: any, pods: models.Pod[], showPodGroupByStatus: boolean) {
+    return (
+        <div className='application-resource-tree__node--lower-section__pod-group__pod-container__pods'>
+            {pods.length !== 0 && showPodGroupByStatus ? (
+                <React.Fragment>
+                    <div className={`pod-view__node__pod pod-view__node__pod--${pods[0].health.toLowerCase()}`}>
+                        <PodHealthIcon state={{status: pods[0].health, message: ''}} key={pods[0].uid} />
+                    </div>
+
+                    <div className='pod-view__node__label--large'>
+                        <a
+                            className='application-resource-tree__node-title'
+                            onClick={() =>
+                                props.onGroupdNodeClick && props.onGroupdNodeClick(node.groupdedNodeIds === 'undefined' ? node.groupdedNodeIds : pods.map(pod => pod.uid))
+                            }>
+                            &nbsp;
+                            <span title={`Click to view the ${pods[0].health.toLowerCase()} pods list`}>
+                                {pods[0].health} {pods.length} pods
+                            </span>
+                        </a>
+                    </div>
+                </React.Fragment>
+            ) : (
+                pods.map(
+                    pod =>
+                        props.nodeMenu && (
+                            <DropDown
+                                key={pod.uid}
+                                isMenu={true}
+                                anchor={() => (
+                                    <Tooltip
+                                        content={
+                                            <div>
+                                                {pod.metadata.name}
+                                                <div>Health: {pod.health}</div>
+                                                {pod.createdAt && (
+                                                    <span>
+                                                        <span>Created: </span>
+                                                        <Moment fromNow={true} ago={true}>
+                                                            {pod.createdAt}
+                                                        </Moment>
+                                                        <span> ago ({<Moment local={true}>{pod.createdAt}</Moment>})</span>
+                                                    </span>
+                                                )}
+                                            </div>
+                                        }
+                                        popperOptions={{
+                                            modifiers: {
+                                                preventOverflow: {
+                                                    enabled: true
+                                                },
+                                                hide: {
+                                                    enabled: false
+                                                },
+                                                flip: {
+                                                    enabled: false
                                                 }
                                             }
-                                        ]}
-                                    />
-                                ))}
-                            </div>
-                            <div className='application-resource-tree__node--lower-section__pod-group__label'>PODS</div>
-                        </div>
-                    </div>
-                )}
-            </div>
+                                        }}
+                                        key={pod.metadata.name}>
+                                        <div style={{position: 'relative'}}>
+                                            {isYoungerThanXMinutes(pod, 30) && (
+                                                <i className='fas fa-star application-resource-tree__node--lower-section__pod-group__pod application-resource-tree__node--lower-section__pod-group__pod__star-icon' />
+                                            )}
+                                            <div
+                                                className={`application-resource-tree__node--lower-section__pod-group__pod application-resource-tree__node--lower-section__pod-group__pod--${pod.health.toLowerCase()}`}>
+                                                <PodHealthIcon state={{status: pod.health, message: ''}} />
+                                            </div>
+                                        </div>
+                                    </Tooltip>
+                                )}>
+                                {() => props.nodeMenu(pod)}
+                            </DropDown>
+                        )
+                )
+            )}
         </div>
     );
 }
@@ -595,6 +654,61 @@ function expandCollapse(node: ResourceTreeNode, props: ApplicationResourceTreePr
     props.setNodeExpansion(node.uid, isExpanded);
 }
 
+function NodeInfoDetails({tag: tag, kind: kind}: {tag: models.InfoItem; kind: string}) {
+    if (kind === 'Pod') {
+        const val = `${tag.name}`;
+        if (val === 'Status Reason') {
+            if (`${tag.value}` !== 'ImagePullBackOff')
+                return (
+                    <span className='application-resource-tree__node-label' title={`Status: ${tag.value}`}>
+                        {tag.value}
+                    </span>
+                );
+            else {
+                return (
+                    <span
+                        className='application-resource-tree__node-label'
+                        title='One of the containers may have the incorrect image name/tag, or you may be fetching from the incorrect repository, or the repository requires authentication.'>
+                        {tag.value}
+                    </span>
+                );
+            }
+        } else if (val === 'Containers') {
+            const arr = `${tag.value}`.split('/');
+            const title = `Number of containers in total: ${arr[1]} \nNumber of ready containers: ${arr[0]}`;
+            return (
+                <span className='application-resource-tree__node-label' title={`${title}`}>
+                    {tag.value}
+                </span>
+            );
+        } else if (val === 'Restart Count') {
+            return (
+                <span className='application-resource-tree__node-label' title={`The total number of restarts of the containers: ${tag.value}`}>
+                    {tag.value}
+                </span>
+            );
+        } else if (val === 'Revision') {
+            return (
+                <span className='application-resource-tree__node-label' title={`The revision in which pod present is: ${tag.value}`}>
+                    {tag.value}
+                </span>
+            );
+        } else {
+            return (
+                <span className='application-resource-tree__node-label' title={`${tag.name}: ${tag.value}`}>
+                    {tag.value}
+                </span>
+            );
+        }
+    } else {
+        return (
+            <span className='application-resource-tree__node-label' title={`${tag.name}: ${tag.value}`}>
+                {tag.value}
+            </span>
+        );
+    }
+}
+
 function renderResourceNode(props: ApplicationResourceTreeProps, id: string, node: ResourceTreeNode & dagre.Node, nodesHavingChildren: Map<string, number>) {
     const fullName = nodeKey(node);
     let comparisonStatus: models.SyncStatusCode = null;
@@ -605,11 +719,8 @@ function renderResourceNode(props: ApplicationResourceTreeProps, id: string, nod
     }
     const appNode = isAppNode(node);
     const rootNode = !node.root;
-    let extLinks: string[] = props.app.status.summary.externalURLs;
+    const extLinks: string[] = props.app.status.summary.externalURLs;
     const childCount = nodesHavingChildren.get(node.uid);
-    if (rootNode) {
-        extLinks = getExternalUrls(props.app.metadata.annotations, props.app.status.summary.externalURLs);
-    }
     return (
         <div
             onClick={() => props.onNodeClick && props.onNodeClick(fullName)}
@@ -672,25 +783,29 @@ function renderResourceNode(props: ApplicationResourceTreeProps, id: string, nod
             </div>
             <div className='application-resource-tree__node-labels'>
                 {node.createdAt || rootNode ? (
-                    <Moment className='application-resource-tree__node-label' fromNow={true} ago={true}>
-                        {node.createdAt || props.app.metadata.creationTimestamp}
-                    </Moment>
+                    <span title={`${node.kind} was created ${moment(node.createdAt).fromNow()}`}>
+                        <Moment className='application-resource-tree__node-label' fromNow={true} ago={true}>
+                            {node.createdAt || props.app.metadata.creationTimestamp}
+                        </Moment>
+                    </span>
                 ) : null}
                 {(node.info || [])
                     .filter(tag => !tag.name.includes('Node'))
                     .slice(0, 4)
-                    .map((tag, i) => (
-                        <span className='application-resource-tree__node-label' title={`${tag.name}:${tag.value}`} key={i}>
-                            {tag.value}
-                        </span>
-                    ))}
+                    .map((tag, i) => {
+                        return <NodeInfoDetails tag={tag} kind={node.kind} key={i} />;
+                    })}
                 {(node.info || []).length > 4 && (
                     <Tooltip
-                        content={(node.info || []).map(i => (
-                            <div key={i.name}>
-                                {i.name}: {i.value}
-                            </div>
-                        ))}
+                        content={
+                            <>
+                                {(node.info || []).map(i => (
+                                    <div key={i.name}>
+                                        {i.name}: {i.value}
+                                    </div>
+                                ))}
+                            </>
+                        }
                         key={node.uid}>
                         <span className='application-resource-tree__node-label' title='More'>
                             More
@@ -743,7 +858,8 @@ export const ApplicationResourceTree = (props: ApplicationResourceTreeProps) =>
         resourceVersion: props.app.metadata.resourceVersion,
         group: 'argoproj.io',
         version: '',
-        children: Array(),
+        // @ts-expect-error its not any
+        children: [],
         status: props.app.status.sync.status,
         health: props.app.status.health,
         uid: props.app.kind + '-' + props.app.metadata.namespace + '-' + props.app.metadata.name,
@@ -783,6 +899,7 @@ export const ApplicationResourceTree = (props: ApplicationResourceTreeProps) =>
     const [filters, setFilters] = React.useState(props.filters);
     const [filteredGraph, setFilteredGraph] = React.useState([]);
     const filteredNodes: any[] = [];
+
     React.useEffect(() => {
         if (props.filters !== filters) {
             setFilters(props.filters);
@@ -790,6 +907,18 @@ export const ApplicationResourceTree = (props: ApplicationResourceTreeProps) =>
             props.setTreeFilterGraph(filteredGraph);
         }
     }, [props.filters]);
+    const {podGroupCount, userMsgs, updateUsrHelpTipMsgs, setShowCompactNodes} = props;
+    const podCount = nodes.filter(node => node.kind === 'Pod').length;
+
+    React.useEffect(() => {
+        if (podCount > podGroupCount) {
+            const userMsg = getUsrMsgKeyToDisplay(appNode.name, 'groupNodes', userMsgs);
+            updateUsrHelpTipMsgs(userMsg);
+            if (!userMsg.display) {
+                setShowCompactNodes(true);
+            }
+        }
+    }, [podCount]);
 
     function filterGraph(app: models.Application, filteredIndicatorParent: string, graphNodesFilter: dagre.graphlib.Graph, predicate: (node: ResourceTreeNode) => boolean) {
         const appKey = appNodeKey(app);
@@ -873,7 +1002,7 @@ export const ApplicationResourceTree = (props: ApplicationResourceTreeProps) =>
                 const loadBalancers = root.networkingInfo.ingress.map(ingress => ingress.hostname || ingress.ip);
                 const colorByService = new Map<string, string>();
                 (childrenByParentKey.get(treeNodeKey(root)) || []).forEach((child, i) => colorByService.set(treeNodeKey(child), TRAFFIC_COLORS[i % TRAFFIC_COLORS.length]));
-                (childrenByParentKey.get(treeNodeKey(root)) || []).sort(compareNodes).forEach((child, i) => {
+                (childrenByParentKey.get(treeNodeKey(root)) || []).sort(compareNodes).forEach(child => {
                     processNode(child, root, [colorByService.get(treeNodeKey(child))]);
                 });
                 if (root.podGroup && props.showCompactNodes) {
