@@ -160,7 +160,7 @@ func writeKeyToFile(keyData string) (string, error) {
 		return "", err
 	}
 
-	err = os.WriteFile(f.Name(), []byte(keyData), 0600)
+	err = os.WriteFile(f.Name(), []byte(keyData), 0o600)
 	if err != nil {
 		os.Remove(f.Name())
 		return "", err
@@ -170,7 +170,7 @@ func writeKeyToFile(keyData string) (string, error) {
 		if err != nil {
 			log.WithFields(log.Fields{
 				common.SecurityField:    common.SecurityMedium,
-				common.SecurityCWEField: 775,
+				common.SecurityCWEField: common.SecurityCWEMissingReleaseOfFileDescriptor,
 			}).Errorf("error closing file %q: %v", f.Name(), err)
 		}
 	}()
@@ -221,7 +221,6 @@ func IsGPGEnabled() bool {
 // InitializeGnuPG will initialize a GnuPG working directory and also create a
 // transient private key so that the trust DB will work correctly.
 func InitializeGnuPG() error {
-
 	gnuPgHome := common.GetGnuPGHomePath()
 
 	// We only operate if ARGOCD_GNUPGHOME is set
@@ -249,13 +248,13 @@ func InitializeGnuPG() error {
 		// re-initialize key ring.
 		err = removeKeyRing(gnuPgHome)
 		if err != nil {
-			return fmt.Errorf("re-initializing keyring at %s failed: %v", gnuPgHome, err)
+			return fmt.Errorf("re-initializing keyring at %s failed: %w", gnuPgHome, err)
 		}
 	}
 
-	err = os.WriteFile(filepath.Join(gnuPgHome, canaryMarkerFilename), []byte("canary"), 0644)
+	err = os.WriteFile(filepath.Join(gnuPgHome, canaryMarkerFilename), []byte("canary"), 0o644)
 	if err != nil {
-		return fmt.Errorf("could not create canary: %v", err)
+		return fmt.Errorf("could not create canary: %w", err)
 	}
 
 	f, err := os.CreateTemp("", "gpg-key-recipe")
@@ -275,7 +274,7 @@ func InitializeGnuPG() error {
 		if err != nil {
 			log.WithFields(log.Fields{
 				common.SecurityField:    common.SecurityMedium,
-				common.SecurityCWEField: 775,
+				common.SecurityCWEField: common.SecurityCWEMissingReleaseOfFileDescriptor,
 			}).Errorf("error closing file %q: %v", f.Name(), err)
 		}
 	}()
@@ -302,7 +301,7 @@ func ImportPGPKeysFromString(keyData string) ([]*appsv1.GnuPGPublicKey, error) {
 		if err != nil {
 			log.WithFields(log.Fields{
 				common.SecurityField:    common.SecurityMedium,
-				common.SecurityCWEField: 775,
+				common.SecurityCWEField: common.SecurityCWEMissingReleaseOfFileDescriptor,
 			}).Errorf("error closing file %q: %v", f.Name(), err)
 		}
 	}()
@@ -430,7 +429,7 @@ func SetPGPTrustLevel(pgpKeys []*appsv1.GnuPGPublicKey, trustLevel string) error
 		if err != nil {
 			log.WithFields(log.Fields{
 				common.SecurityField:    common.SecurityMedium,
-				common.SecurityCWEField: 775,
+				common.SecurityCWEField: common.SecurityCWEMissingReleaseOfFileDescriptor,
 			}).Errorf("error closing file %q: %v", f.Name(), err)
 		}
 	}()
@@ -674,7 +673,7 @@ func ParseGitCommitVerification(signature string) PGPVerifyResult {
 	}
 
 	if parseOk && linesParsed < MaxVerificationLinesToParse {
-		// Operation successfull - return result
+		// Operation successful - return result
 		return result
 	} else if linesParsed >= MaxVerificationLinesToParse {
 		// Too many output lines, return error
@@ -696,7 +695,6 @@ func SyncKeyRingFromDirectory(basePath string) ([]string, []string, error) {
 	fingerprints := make([]string, 0)
 	removedKeys := make([]string, 0)
 	st, err := os.Stat(basePath)
-
 	if err != nil {
 		return nil, nil, err
 	}
@@ -718,14 +716,14 @@ func SyncKeyRingFromDirectory(basePath string) ([]string, []string, error) {
 		return nil
 	})
 	if err != nil {
-		return nil, nil, err
+		return nil, nil, fmt.Errorf("error walk path: %w", err)
 	}
 
 	// Collect GPG keys installed in the key ring
 	installed := make(map[string]*appsv1.GnuPGPublicKey)
 	keys, err := GetInstalledPGPKeys(nil)
 	if err != nil {
-		return nil, nil, err
+		return nil, nil, fmt.Errorf("error get installed PGP keys: %w", err)
 	}
 	for _, v := range keys {
 		installed[v.KeyID] = v
@@ -736,16 +734,16 @@ func SyncKeyRingFromDirectory(basePath string) ([]string, []string, error) {
 		if _, ok := installed[key]; !ok {
 			addedKey, err := ImportPGPKeys(path.Join(basePath, key))
 			if err != nil {
-				return nil, nil, err
+				return nil, nil, fmt.Errorf("error import PGP keys: %w", err)
 			}
 			if len(addedKey) != 1 {
-				return nil, nil, fmt.Errorf("Invalid key found in %s", path.Join(basePath, key))
+				return nil, nil, fmt.Errorf("invalid key found in %s", path.Join(basePath, key))
 			}
 			importedKey, err := GetInstalledPGPKeys([]string{addedKey[0].KeyID})
 			if err != nil {
-				return nil, nil, err
+				return nil, nil, fmt.Errorf("error get installed PGP keys: %w", err)
 			} else if len(importedKey) != 1 {
-				return nil, nil, fmt.Errorf("Could not get details of imported key ID %s", importedKey)
+				return nil, nil, fmt.Errorf("could not get details of imported key ID %s", importedKey)
 			}
 			newKeys = append(newKeys, key)
 			fingerprints = append(fingerprints, importedKey[0].Fingerprint)
@@ -756,12 +754,12 @@ func SyncKeyRingFromDirectory(basePath string) ([]string, []string, error) {
 	for key := range installed {
 		secret, err := IsSecretKey(key)
 		if err != nil {
-			return nil, nil, err
+			return nil, nil, fmt.Errorf("error check secret key: %w", err)
 		}
 		if _, ok := configured[key]; !ok && !secret {
 			err := DeletePGPKey(key)
 			if err != nil {
-				return nil, nil, err
+				return nil, nil, fmt.Errorf("error delete PGP keys: %w", err)
 			}
 			removedKeys = append(removedKeys, key)
 		}
@@ -772,5 +770,5 @@ func SyncKeyRingFromDirectory(basePath string) ([]string, []string, error) {
 		_ = SetPGPTrustLevelById(fingerprints, TrustUltimate)
 	}
 
-	return newKeys, removedKeys, err
+	return newKeys, removedKeys, nil
 }
